@@ -4,7 +4,7 @@
  *
  * Protocol module stuff.
  *
- * $Id: pmodule.h 4813 2006-02-10 15:02:54Z nenolod $
+ * $Id: pmodule.h 5724 2006-07-04 16:06:20Z w00t $
  */
 
 #ifndef PMODULE_H
@@ -27,6 +27,7 @@ E pcommand_t *pcommand_find(char *token);
 
 E boolean_t pmodule_loaded;
 E boolean_t backend_loaded;
+E int authservice_loaded;
 
 /*  -- what the HELL are these used for? A grep reveals nothing.. --w00t
  *  -- they are used to provide a hint to third-party module coders about what
@@ -39,7 +40,7 @@ E boolean_t backend_loaded;
 #define PROTOCOL_HYPERION		5
 #define PROTOCOL_INSPIRCD		6
 #define PROTOCOL_IRCNET			7
-#define PROTOCOL_MONKEY			8
+#define PROTOCOL_MONKEY			8 /* obsolete */
 #define PROTOCOL_PLEXUS			9
 #define PROTOCOL_PTLINK			10
 #define PROTOCOL_RATBOX			11
