@@ -4,7 +4,7 @@
  *
  * This code contains the channel mode definitions for InspIRCd.
  *
- * $Id: inspircd.h 4639 2006-01-21 22:06:41Z jilles $
+ * $Id: inspircd.h 5420 2006-06-18 19:02:46Z jilles $
  */
 
 #ifndef RATBOX_H
@@ -16,7 +16,7 @@
 #define CMODE_MODREG	0x00002000	/* bahamut +M */
 #define CMODE_REGONLY	0x00004000	/* bahamut +R */
 #define CMODE_OPERONLY  0x00008000      /* bahamut +O */
-#define CMODE_ADMONLY	0x00010000	/* unreal +A */
+#define CMODE_NONOTICE	0x00010000	/* inspircd +T */
 #define CMODE_PEACE	0x00020000	/* unreal +Q */
 #define CMODE_STRIP	0x00040000	/* unreal +S */
 #define CMODE_NOKNOCK	0x00080000	/* unreal +K */
@@ -25,6 +25,8 @@
 #define CMODE_HIDING	0x00400000	/* unreal +u */
 #define CMODE_SSLONLY	0x00800000	/* unreal +z */
 #define CMODE_STICKY	0x01000000	/* unreal +N */
+#define CMODE_CENSOR    0x02000000	/* Censor +G */
+#define CMODE_NOCAPS    0x04000000	/* No capitals */
 
 #define CMODE_OWNER     0x10000000      /* unreal +q */
 #define CMODE_PROTECT   0x20000000      /* unreal +a */
