@@ -4,7 +4,7 @@
  *
  * This file contains code for the CService INFO functions.
  *
- * $Id: info.c 5073 2006-04-14 11:16:18Z w00t $
+ * $Id: info.c 5686 2006-07-03 16:25:03Z jilles $
  */
 
 #include "atheme.h"
@@ -12,7 +12,7 @@
 DECLARE_MODULE_V1
 (
 	"chanserv/info", FALSE, _modinit, _moddeinit,
-	"$Id: info.c 5073 2006-04-14 11:16:18Z w00t $",
+	"$Id: info.c 5686 2006-07-03 16:25:03Z jilles $",
 	"Atheme Development Group <http://www.atheme.org>"
 );
 
@@ -26,8 +26,8 @@ list_t *cs_helptree;
 
 void _modinit(module_t *m)
 {
-	cs_cmdtree = module_locate_symbol("chanserv/main", "cs_cmdtree");
-	cs_helptree = module_locate_symbol("chanserv/main", "cs_helptree");
+	MODULE_USE_SYMBOL(cs_cmdtree, "chanserv/main", "cs_cmdtree");
+	MODULE_USE_SYMBOL(cs_helptree, "chanserv/main", "cs_helptree");
 
         command_add(&cs_info, cs_cmdtree);
 	help_addentry(cs_helptree, "INFO", "help/cservice/info", NULL);
@@ -48,6 +48,8 @@ static void cs_cmd_info(char *origin)
 	struct tm tm;
 	metadata_t *md;
 	hook_channel_req_t req;
+	char *p, *q, *qq;
+	int dir;
 
 	if (!name)
 	{
@@ -87,24 +89,41 @@ static void cs_cmd_info(char *origin)
 
 	notice(chansvs.nick, origin, "Registered : %s (%s ago)", strfbuf, time_ago(mc->registered));
 
-	if (mc->mlock_on || mc->mlock_off || mc->mlock_limit || mc->mlock_key)
+	if (CURRTIME - mc->used >= 86400)
+	{
+		tm = *localtime(&mc->used);
+		strftime(strfbuf, sizeof(strfbuf) - 1, "%b %d %H:%M:%S %Y", &tm);
+		notice(chansvs.nick, origin, "Last used  : %s (%s ago)", strfbuf, time_ago(mc->used));
+	}
+
+	md = metadata_find(mc, METADATA_CHANNEL, "private:mlockext");
+	if (mc->mlock_on || mc->mlock_off || mc->mlock_limit || mc->mlock_key || md)
 	{
 		char params[BUFSIZE];
 
+		if (md != NULL && strlen(md->value) > 450)
+		{
+			/* Be safe */
+			notice(chansvs.nick, origin, "Mode lock is too long, not entirely shown");
+			md = NULL;
+		}
+
 		*buf = 0;
 		*params = 0;
+		dir = MTYPE_NUL;
 
 		if (mc->mlock_on)
 		{
-			strcat(buf, "+");
+			if (dir != MTYPE_ADD)
+				dir = MTYPE_ADD, strcat(buf, "+");
 			strcat(buf, flags_to_string(mc->mlock_on));
 
 		}
 
 		if (mc->mlock_limit)
 		{
-			if (*buf == '\0')
-				strcat(buf, "+");
+			if (dir != MTYPE_ADD)
+				dir = MTYPE_ADD, strcat(buf, "+");
 			strcat(buf, "l");
 			strcat(params, " ");
 			strcat(params, itoa(mc->mlock_limit));
@@ -112,14 +131,45 @@ static void cs_cmd_info(char *origin)
 
 		if (mc->mlock_key)
 		{
-			if (*buf == '\0')
-				strcat(buf, "+");
+			if (dir != MTYPE_ADD)
+				dir = MTYPE_ADD, strcat(buf, "+");
 			strcat(buf, "k");
+			strcat(params, " *");
+		}
+
+		if (md)
+		{
+			p = md->value;
+			q = buf + strlen(buf);
+			while (*p != '\0')
+			{
+				if (p[1] != ' ' && p[1] != '\0')
+				{
+					if (dir != MTYPE_ADD)
+						dir = MTYPE_ADD, *q++ = '+';
+					*q++ = *p++;
+					strcat(params, " ");
+					qq = params + strlen(params);
+					while (*p != '\0' && *p != ' ')
+						*qq++ = *p++;
+					*qq = '\0';
+				}
+				else
+				{
+					p++;
+					while (*p != '\0' && *p != ' ')
+						p++;
+				}
+				while (*p == ' ')
+					p++;
+			}
+			*q = '\0';
 		}
 
 		if (mc->mlock_off)
 		{
-			strcat(buf, "-");
+			if (dir != MTYPE_DEL)
+				dir = MTYPE_DEL, strcat(buf, "-");
 			strcat(buf, flags_to_string(mc->mlock_off));
 			if (mc->mlock_off & CMODE_LIMIT)
 				strcat(buf, "l");
@@ -127,6 +177,27 @@ static void cs_cmd_info(char *origin)
 				strcat(buf, "k");
 		}
 
+		if (md)
+		{
+			p = md->value;
+			q = buf + strlen(buf);
+			while (*p != '\0')
+			{
+				if (p[1] == ' ' || p[1] == '\0')
+				{
+					if (dir != MTYPE_DEL)
+						dir = MTYPE_DEL, *q++ = '-';
+					*q++ = *p;
+				}
+				p++;
+				while (*p != '\0' && *p != ' ')
+					p++;
+				while (*p == ' ')
+					p++;
+			}
+			*q = '\0';
+		}
+
 		if (*buf)
 			notice(chansvs.nick, origin, "Mode lock  : %s%s", buf, (params) ? params : "");
 	}
