@@ -4,7 +4,7 @@
  *
  * VHost management! (ratbox only right now.)
  *
- * $Id: vhost.c 5113 2006-04-19 11:58:25Z jilles $
+ * $Id: vhost.c 5686 2006-07-03 16:25:03Z jilles $
  */
 
 #include "atheme.h"
@@ -12,7 +12,7 @@
 DECLARE_MODULE_V1
 (
 	"nickserv/vhost", FALSE, _modinit, _moddeinit,
-	"$Id: vhost.c 5113 2006-04-19 11:58:25Z jilles $",
+	"$Id: vhost.c 5686 2006-07-03 16:25:03Z jilles $",
 	"Atheme Development Group <http://www.atheme.org>"
 );
 
@@ -26,11 +26,12 @@ command_t ns_vhost = { "VHOST", "Manages user virtualhosts.",
 
 void _modinit(module_t *m)
 {
-	ns_cmdtree = module_locate_symbol("nickserv/main", "ns_cmdtree");
+	MODULE_USE_SYMBOL(ns_cmdtree, "nickserv/main", "ns_cmdtree");
+	MODULE_USE_SYMBOL(ns_helptree, "nickserv/main", "ns_helptree");
+
 	hook_add_event("user_identify");
 	hook_add_hook("user_identify", vhost_on_identify);
 	command_add(&ns_vhost, ns_cmdtree);
-	ns_helptree = module_locate_symbol("nickserv/main", "ns_helptree");
 	help_addentry(ns_helptree, "VHOST", "help/nickserv/vhost", NULL);
 }
 
