@@ -4,7 +4,7 @@
  *
  * Module listing.
  *
- * $Id: modlist.c 4613 2006-01-19 23:52:30Z jilles $
+ * $Id: modlist.c 5686 2006-07-03 16:25:03Z jilles $
  */
 
 #include "atheme.h"
@@ -12,7 +12,7 @@
 DECLARE_MODULE_V1
 (
 	"operserv/modlist", FALSE, _modinit, _moddeinit,
-	"$Id: modlist.c 4613 2006-01-19 23:52:30Z jilles $",
+	"$Id: modlist.c 5686 2006-07-03 16:25:03Z jilles $",
 	"Atheme Development Group <http://www.atheme.org>"
 );
 
@@ -27,8 +27,9 @@ extern list_t modules;
 
 void _modinit(module_t *m)
 {
-	os_cmdtree = module_locate_symbol("operserv/main", "os_cmdtree");
-	os_helptree = module_locate_symbol("operserv/main", "os_helptree");
+	MODULE_USE_SYMBOL(os_cmdtree, "operserv/main", "os_cmdtree");
+	MODULE_USE_SYMBOL(os_helptree, "operserv/main", "os_helptree");
+
 	command_add(&os_modlist, os_cmdtree);
 	help_addentry(os_helptree, "MODLIST", "help/oservice/modlist", NULL);
 }
