@@ -2,15 +2,15 @@
  * Copyright (c) 2003-2004 E. Will et al.
  * Rights to this code are documented in doc/LICENSE.
  *
- * This file contains protocol support for ptlink-based ircd.
+ * This file contains protocol support for ptlink ircd.
  *
- * $Id: ptlink.c 5131 2006-04-29 19:09:24Z jilles $
+ * $Id: ptlink.c 5628 2006-07-01 23:38:42Z jilles $
  */
 
 #include "atheme.h"
 #include "protocol/ptlink.h"
 
-DECLARE_MODULE_V1("protocol/ptlink", TRUE, _modinit, NULL, "$Id: ptlink.c 5131 2006-04-29 19:09:24Z jilles $", "Atheme Development Group <http://www.atheme.org>");
+DECLARE_MODULE_V1("protocol/ptlink", TRUE, _modinit, NULL, "$Id: ptlink.c 5628 2006-07-01 23:38:42Z jilles $", "Atheme Development Group <http://www.atheme.org>");
 
 /* *INDENT-OFF* */
 
@@ -23,7 +23,7 @@ ircd_t PTLink = {
         FALSE,                          /* Whether or not we support channel protection. */
         FALSE,                          /* Whether or not we support halfops. */
 	FALSE,				/* Whether or not we use P10 */
-	FALSE,				/* Whether or not we use vHosts. */
+	TRUE,				/* Whether or not we use vHosts. */
 	0,				/* Oper-only cmodes */
         0,                              /* Integer flag for owner channel flag. */
         0,                              /* Integer flag for protect channel flag. */
@@ -33,9 +33,9 @@ ircd_t PTLink = {
         "+",                            /* Mode we set for halfops. */
 	PROTOCOL_PTLINK,		/* Protocol type */
 	0,                              /* Permanent cmodes */
-	"beI",                          /* Ban-like cmodes */
-	'e',                            /* Except mchar */
-	'I'                             /* Invex mchar */
+	"b",                            /* Ban-like cmodes */
+	0,                              /* Except mchar */
+	0                               /* Invex mchar */
 };
 
 struct cmode_ ptlink_mode_list[] = {
@@ -57,25 +57,55 @@ struct cmode_ ptlink_mode_list[] = {
   { '\0', 0 }
 };
 
-struct cmode_ ptlink_ignore_mode_list[] = {
-  { 'f', CMODE_FLOOD  },
+static boolean_t check_flood(const char *, channel_t *, mychan_t *, user_t *, myuser_t *);
+
+struct extmode ptlink_ignore_mode_list[] = {
+  { 'f', check_flood  },
   { '\0', 0 }
 };
 
 struct cmode_ ptlink_status_mode_list[] = {
+  { 'a', CMODE_PROTECT},
   { 'o', CMODE_OP    },
+  { 'h', CMODE_HALFOP},
   { 'v', CMODE_VOICE },
   { '\0', 0 }
 };
 
 struct cmode_ ptlink_prefix_mode_list[] = {
+  { '.', CMODE_PROTECT},
   { '@', CMODE_OP    },
+  { '%', CMODE_HALFOP},
   { '+', CMODE_VOICE },
   { '\0', 0 }
 };
 
 /* *INDENT-ON* */
 
+static boolean_t check_flood(const char *value, channel_t *c, mychan_t *mc, user_t *u, myuser_t *mu)
+{
+	const char *p, *arg2;
+
+	p = value, arg2 = NULL;
+	while (*p != '\0')
+	{
+		if (*p == ':')
+		{
+			if (arg2 != NULL)
+				return FALSE;
+			arg2 = p + 1;
+		}
+		else if (!isdigit(*p))
+			return FALSE;
+		p++;
+	}
+	if (arg2 == NULL)
+		return FALSE;
+	if (p - arg2 > 2 || arg2 - value - 1 > 2 || atoi(value) <= 1 || !atoi(arg2))
+		return FALSE;
+	return TRUE;
+}
+
 /* login to our uplink */
 static uint8_t ptlink_server_login(void)
 {
@@ -88,8 +118,8 @@ static uint8_t ptlink_server_login(void)
 	me.bursting = TRUE;
 
 	sts("CAPAB :QS PTS4");
-	sts("SERVER %s 1 Atheme :%s", me.name, me.desc);
-	sts("SVINFO 6 3 0 :%ld", CURRTIME);
+	sts("SERVER %s 1 Atheme-%s :%s", me.name, version, me.desc);
+	sts("SVINFO 10 3 0 :%ld", CURRTIME);
 
 	services_init();
 
@@ -99,7 +129,7 @@ static uint8_t ptlink_server_login(void)
 /* introduce a client */
 static void ptlink_introduce_nick(char *nick, char *user, char *host, char *real, char *uid)
 {
-	sts("NICK %s 1 %ld +%s %s %s %s :%s", nick, CURRTIME, "io", user, host, me.name, real);
+	sts("NICK %s 1 %ld +%sp %s %s %s %s :%s", nick, CURRTIME, "io", user, host, host, me.name, real);
 }
 
 /* invite a user to a channel */
@@ -233,7 +263,8 @@ static void ptlink_kline_sts(char *server, char *user, char *host, long duration
 	if (!me.connected)
 		return;
 
-	sts(":%s GLINE %s@%s %ld :%s", opersvs.nick, user, host, duration, reason);
+	/* we don't know the real setter here :( */
+	sts(":%s GLINE %s@%s %ld %s :%s", opersvs.nick, user, host, duration, opersvs.nick, reason);
 }
 
 /* server-to-server UNKLINE wrapper */
@@ -284,7 +315,7 @@ static void ptlink_on_login(char *origin, char *user, char *wantedhost)
 	if (nicksvs.me == NULL || irccasecmp(origin, user))
 		return;
 
-	sts(":%s SVSMODE %s +rd %ld", me.name, origin, CURRTIME);
+	sts(":%s SVSMODE %s +r", me.name, origin);
 }
 
 /* protocol-specific stuff to do on login */
@@ -296,7 +327,7 @@ static boolean_t ptlink_on_logout(char *origin, char *user, char *wantedhost)
 	if (nicksvs.me == NULL || irccasecmp(origin, user))
 		return FALSE;
 
-	sts(":%s SVSMODE %s -r+d %ld", me.name, origin, CURRTIME);
+	sts(":%s SVSMODE %s -r", me.name, origin);
 	return FALSE;
 }
 
@@ -306,7 +337,17 @@ static void ptlink_jupe(char *server, char *reason)
 		return;
 
 	sts(":%s SQUIT %s :%s", opersvs.nick, server, reason);
-	sts(":%s SERVER %s 2 Atheme :%s", me.name, server, reason);
+	sts(":%s SERVER %s 2 Atheme-%s-jupe :%s", me.name, server, version, reason);
+}
+
+static void ptlink_sethost_sts(char *source, char *target, char *host)
+{
+	user_t *tu = user_find(target);
+
+	if (!tu)
+		return;
+
+	sts(":%s NEWMASK %s :%s", me.name, host, tu->nick);
 }
 
 static void m_topic(char *origin, uint8_t parc, char *parv[])
@@ -331,8 +372,16 @@ static void m_pong(char *origin, uint8_t parc, char *parv[])
 	server_t *s;
 
 	/* someone replied to our PING */
+	if (!parv[0])
+		return;
+	s = server_find(parv[0]);
+	if (s == NULL)
+		return;
+	handle_eob(s);
+  
 	if (irccasecmp(me.actual, parv[0]))
 		return;
+
 	me.uplinkpong = CURRTIME;
 
 	/* -> :test.projectxero.net PONG test.projectxero.net :shrike.malkier.net */
@@ -374,8 +423,6 @@ static void m_sjoin(char *origin, uint8_t parc, char *parv[])
 	/* -> :proteus.malkier.net SJOIN 1073516550 #shrike +tn :@sycobuny @+rakaur */
 
 	channel_t *c;
-	uint8_t modec = 0;
-	char *modev[16];
 	uint8_t userc;
 	char *userv[256];
 	uint8_t i;
@@ -384,13 +431,6 @@ static void m_sjoin(char *origin, uint8_t parc, char *parv[])
 	if (origin)
 	{
 		/* :origin SJOIN ts chan modestr [key or limits] :users */
-		modev[modec++] = parv[2];
-
-		if (parc > 4)
-			modev[modec++] = parv[3];
-		if (parc > 5)
-			modev[modec++] = parv[4];
-
 		c = channel_find(parv[1]);
 		ts = atol(parv[0]);
 
@@ -411,11 +451,7 @@ static void m_sjoin(char *origin, uint8_t parc, char *parv[])
 			 * and set the new TS.
 			 */
 
-			c->modes = 0;
-			c->limit = 0;
-			if (c->key)
-				free(c->key);
-			c->key = NULL;
+			clear_simple_modes(c);
 
 			LIST_FOREACH(n, c->members.head)
 			{
@@ -434,9 +470,10 @@ static void m_sjoin(char *origin, uint8_t parc, char *parv[])
 			slog(LG_INFO, "m_sjoin(): TS changed for %s (%ld -> %ld)", c->name, c->ts, ts);
 
 			c->ts = ts;
+			hook_call_event("channel_tschange", c);
 		}
 
-		channel_mode(NULL, c, modec, modev);
+		channel_mode(NULL, c, parc - 3, parv + 2);
 
 		userc = sjtoken(parv[parc - 1], ' ', userv);
 
@@ -447,9 +484,19 @@ static void m_sjoin(char *origin, uint8_t parc, char *parv[])
 
 static void m_part(char *origin, uint8_t parc, char *parv[])
 {
-	slog(LG_DEBUG, "m_part(): user left channel: %s -> %s", origin, parv[0]);
+	uint8_t chanc;
+	char *chanv[256];
+	int i;
 
-	chanuser_delete(channel_find(parv[0]), user_find(origin));
+	if (parc < 1)
+		return;
+	chanc = sjtoken(parv[0], ',', chanv);
+	for (i = 0; i < chanc; i++)
+	{
+		slog(LG_DEBUG, "m_part(): user left channel: %s -> %s", origin, chanv[i]);
+
+		chanuser_delete(channel_find(chanv[i]), user_find(origin));
+	}
 }
 
 static void m_nick(char *origin, uint8_t parc, char *parv[])
@@ -458,9 +505,9 @@ static void m_nick(char *origin, uint8_t parc, char *parv[])
 	user_t *u;
 
 	/* got the right number of args for an introduction? */
-	if (parc == 8)
+	if (parc == 9)
 	{
-		s = server_find(parv[6]);
+		s = server_find(parv[7]);
 		if (!s)
 		{
 			slog(LG_DEBUG, "m_nick(): new user on nonexistant server: %s", parv[6]);
@@ -469,7 +516,7 @@ static void m_nick(char *origin, uint8_t parc, char *parv[])
 
 		slog(LG_DEBUG, "m_nick(): new user on `%s': %s", s->name, parv[0]);
 
-		u = user_add(parv[0], parv[4], parv[5], NULL, NULL, NULL, parv[7], s, atoi(parv[0]));
+		u = user_add(parv[0], parv[4], parv[5], parv[6], NULL, NULL, parv[8], s, atoi(parv[0]));
 
 		user_mode(u, parv[3]);
 
@@ -507,7 +554,7 @@ static void m_nick(char *origin, uint8_t parc, char *parv[])
 		/* fix up +r if necessary -- jilles */
 		if (nicksvs.me != NULL && u->myuser != NULL && !(u->myuser->flags & MU_WAITAUTH) && irccasecmp(u->nick, parv[0]) && !irccasecmp(parv[0], u->myuser->name))
 			/* changed nick to registered one, reset +r */
-			sts(":%s SVSMODE %s +rd %ld", me.name, parv[0], time(NULL));
+			sts(":%s SVSMODE %s +r", me.name, parv[0]);
 
 		/* remove the current one from the list */
 		n = node_find(u, &userlist[u->hash]);
@@ -619,6 +666,13 @@ static void m_server(char *origin, uint8_t parc, char *parv[])
 
 	if (cnt.server == 2)
 		me.actual = sstrdup(parv[0]);
+	else
+	{
+		/* elicit PONG for EOB detection; pinging uplink is
+		 * already done elsewhere -- jilles
+		 */
+		sts(":%s PING %s %s", me.name, me.name, parv[0]);
+	}
 }
 
 static void m_stats(char *origin, uint8_t parc, char *parv[])
@@ -685,6 +739,30 @@ static void m_error(char *origin, uint8_t parc, char *parv[])
 	slog(LG_INFO, "m_error(): error from server: %s", parv[0]);
 }
 
+static void m_newmask(char *origin, uint8_t parc, char *parv[])
+{
+	user_t *target;
+	char *p;
+
+	if (parc < 1)
+		return;
+	target = user_find(parc >= 2 ? parv[1] : origin);
+	p = strchr(parv[0], '@');
+	if (p != NULL)
+	{
+		strlcpy(target->vhost, p + 1, sizeof target->vhost);
+		if (p - parv[0] < sizeof target->user && p > parv[0])
+		{
+			memcpy(target->user, parv[0], p - parv[0]);
+			target->user[p - parv[0]] = '\0';
+		}
+	}
+	else
+		strlcpy(target->vhost, parv[0], sizeof target->vhost);
+	slog(LG_DEBUG, "m_newmask(): %s -> %s@%s",
+			target->nick, target->user, target->vhost);
+}
+
 static void m_motd(char *origin, uint8_t parc, char *parv[])
 {
 	handle_motd(user_find(origin));
@@ -713,6 +791,7 @@ void _modinit(module_t * m)
 	ircd_on_logout = &ptlink_on_logout;
 	jupe = &ptlink_jupe;
 	invite_sts = &ptlink_invite_sts;
+	sethost_sts = &ptlink_sethost_sts;
 
 	mode_list = ptlink_mode_list;
 	ignore_mode_list = ptlink_ignore_mode_list;
@@ -726,8 +805,10 @@ void _modinit(module_t * m)
 	pcommand_add("PRIVMSG", m_privmsg);
 	pcommand_add("NOTICE", m_notice);
 	pcommand_add("SJOIN", m_sjoin);
+	pcommand_add("NJOIN", m_sjoin);
 	pcommand_add("PART", m_part);
 	pcommand_add("NICK", m_nick);
+	pcommand_add("NNICK", m_nick);
 	pcommand_add("QUIT", m_quit);
 	pcommand_add("MODE", m_mode);
 	pcommand_add("KICK", m_kick);
@@ -744,6 +825,7 @@ void _modinit(module_t * m)
 	pcommand_add("PASS", m_pass);
 	pcommand_add("ERROR", m_error);
 	pcommand_add("TOPIC", m_topic);
+	pcommand_add("NEWMASK", m_newmask);
 	pcommand_add("MOTD", m_motd);
 
 	m->mflags = MODTYPE_CORE;
