@@ -4,13 +4,13 @@
  *
  * This file contains protocol support for Ultimate3 ircd.
  *
- * $Id: ultimate3.c 5131 2006-04-29 19:09:24Z jilles $
+ * $Id: ultimate3.c 5628 2006-07-01 23:38:42Z jilles $
  */
 
 #include "atheme.h"
 #include "protocol/ultimate3.h"
 
-DECLARE_MODULE_V1("protocol/ultimate3", TRUE, _modinit, NULL, "$Id: ultimate3.c 5131 2006-04-29 19:09:24Z jilles $", "Atheme Development Group <http://www.atheme.org>");
+DECLARE_MODULE_V1("protocol/ultimate3", TRUE, _modinit, NULL, "$Id: ultimate3.c 5628 2006-07-01 23:38:42Z jilles $", "Atheme Development Group <http://www.atheme.org>");
 
 /* *INDENT-OFF* */
 
@@ -51,7 +51,7 @@ struct cmode_ ultimate3_mode_list[] = {
   { '\0', 0 }
 };
 
-struct cmode_ ultimate3_ignore_mode_list[] = {
+struct extmode ultimate3_ignore_mode_list[] = {
   { '\0', 0 }
 };
 
@@ -173,9 +173,8 @@ static void ultimate3_notice(char *from, char *target, char *fmt, ...)
 	va_list ap;
 	char buf[BUFSIZE];
 
-	/*
-	 * ultimate3 appears to not like it when it recieves
-	 * * a message to and from the same person.
+	/* ultimate3 appears to not like it when it recieves
+	 * a message to and from the same person.
 	 */
 	if (!strcasecmp(from, target))
 		return;
@@ -335,25 +334,28 @@ static void m_topic(char *origin, uint8_t parc, char *parv[])
 
 static void m_ping(char *origin, uint8_t parc, char *parv[])
 {
-	/*
-	 * reply to PING's 
-	 */
+	/* reply to PING's */
 	sts(":%s PONG %s %s", me.name, me.name, parv[0]);
 }
 
 static void m_pong(char *origin, uint8_t parc, char *parv[])
 {
-	/*
-	 * someone replied to our PING 
-	 */
-	if ((!parv[0]) || (strcasecmp(me.actual, parv[0])))
+	server_t *s;
+
+	/* someone replied to our PING */
+	if (!parv[0])
+		return;
+	s = server_find(parv[0]);
+	if (s == NULL)
+		return;
+	handle_eob(s);
+
+	if (irccasecmp(me.actual, parv[0]))
 		return;
 
 	me.uplinkpong = CURRTIME;
 
-	/*
-	 * -> :test.projectxero.net PONG test.projectxero.net :shrike.malkier.net 
-	 */
+	/* -> :test.projectxero.net PONG test.projectxero.net :shrike.malkier.net */
 	if (me.bursting)
 	{
 #ifdef HAVE_GETTIMEOFDAY
@@ -396,8 +398,6 @@ static void m_sjoin(char *origin, uint8_t parc, char *parv[])
 	 */
 
 	channel_t *c;
-	uint8_t modec = 0;
-	char *modev[16];
 	uint8_t userc;
 	char *userv[256];
 	uint8_t i;
@@ -405,16 +405,7 @@ static void m_sjoin(char *origin, uint8_t parc, char *parv[])
 
 	if (origin && parc >= 4)
 	{
-		/*
-		 * :origin SJOIN ts chan modestr [key or limits] :users 
-		 */
-		modev[modec++] = parv[2];
-
-		if (parc > 4)
-			modev[modec++] = parv[3];
-		if (parc > 5)
-			modev[modec++] = parv[4];
-
+		/* :origin SJOIN ts chan modestr [key or limits] :users */
 		c = channel_find(parv[1]);
 		ts = atol(parv[0]);
 
@@ -431,25 +422,19 @@ static void m_sjoin(char *origin, uint8_t parc, char *parv[])
 
 			/*
 			 * the TS changed.  a TS change requires the following things
-			 * * to be done to the channel:  reset all modes to nothing, remove
-			 * * all status modes on known users on the channel (including ours),
-			 * * and set the new TS.
+			 * to be done to the channel:  reset all modes to nothing, remove
+			 * all status modes on known users on the channel (including ours),
+			 * and set the new TS.
 			 */
 
-			c->modes = 0;
-			c->limit = 0;
-			if (c->key)
-				free(c->key);
-			c->key = NULL;
+			clear_simple_modes(c);
 
 			LIST_FOREACH(n, c->members.head)
 			{
 				cu = (chanuser_t *)n->data;
 				if (cu->user->server == me.me)
 				{
-					/*
-					 * it's a service, reop 
-					 */
+					/* it's a service, reop */
 					sts(":%s PART %s :Reop", cu->user->nick, c->name);
 					sts(":%s SJOIN %ld %s + :@%s", me.name, ts, c->name, cu->user->nick);
 					cu->modes = CMODE_OP;
@@ -461,9 +446,10 @@ static void m_sjoin(char *origin, uint8_t parc, char *parv[])
 			slog(LG_INFO, "m_sjoin(): TS changed for %s (%ld -> %ld)", c->name, c->ts, ts);
 
 			c->ts = ts;
+			hook_call_event("channel_tschange", c);
 		}
 
-		channel_mode(NULL, c, modec, modev);
+		channel_mode(NULL, c, parc - 3, parv + 2);
 
 		userc = sjtoken(parv[parc - 1], ' ', userv);
 
@@ -482,25 +468,19 @@ static void m_sjoin(char *origin, uint8_t parc, char *parv[])
 
 			/*
 			 * the TS changed.  a TS change requires the following things
-			 * * to be done to the channel:  reset all modes to nothing, remove
-			 * * all status modes on known users on the channel (including ours),
-			 * * and set the new TS.
+			 * to be done to the channel:  reset all modes to nothing, remove
+			 * all status modes on known users on the channel (including ours),
+			 * and set the new TS.
 			 */
 
-			c->modes = 0;
-			c->limit = 0;
-			if (c->key)
-				free(c->key);
-			c->key = NULL;
+			clear_simple_modes(c);
 
 			LIST_FOREACH(n, c->members.head)
 			{
 				cu = (chanuser_t *)n->data;
 				if (cu->user->server == me.me)
 				{
-					/*
-					 * it's a service, reop 
-					 */
+					/* it's a service, reop */
 					sts(":%s PART %s :Reop", cu->user->nick, c->name);
 					sts(":%s SJOIN %ld %s + :@%s", me.name, ts, c->name, cu->user->nick);
 					cu->modes = CMODE_OP;
@@ -512,6 +492,7 @@ static void m_sjoin(char *origin, uint8_t parc, char *parv[])
 			slog(LG_INFO, "m_sjoin(): TS changed for %s (%ld -> %ld)", c->name, c->ts, ts);
 
 			c->ts = ts;
+			hook_call_event("channel_tschange", c);
 		}
 
 		chanuser_add(c, origin);
@@ -520,9 +501,19 @@ static void m_sjoin(char *origin, uint8_t parc, char *parv[])
 
 static void m_part(char *origin, uint8_t parc, char *parv[])
 {
-	slog(LG_DEBUG, "m_part(): user left channel: %s -> %s", origin, parv[0]);
+	uint8_t chanc;
+	char *chanv[256];
+	int i;
 
-	chanuser_delete(channel_find(parv[0]), user_find(origin));
+	if (parc < 1)
+		return;
+	chanc = sjtoken(parv[0], ',', chanv);
+	for (i = 0; i < chanc; i++)
+	{
+		slog(LG_DEBUG, "m_part(): user left channel: %s -> %s", origin, chanv[i]);
+
+		chanuser_delete(channel_find(chanv[i]), user_find(origin));
+	}
 }
 
 static void m_nick(char *origin, uint8_t parc, char *parv[])
@@ -550,23 +541,19 @@ static void m_nick(char *origin, uint8_t parc, char *parv[])
 #else
 		u = user_add(parv[0], parv[5], parv[6], parv[7], inet_ntoa(addr), NULL, parv[11], s, atoi(parv[2]));
 #endif
-		/*
-		 * user modes 
-		 */
+		/* user modes */
 		user_mode(u, parv[3]);
-		/*
-		 * Oper Mode 
-		 */
+		/* Oper Mode */
 		user_mode(u, parv[4]);
 
 		/*
 		 * Ok, we have the user ready to go.
-		 * * Here's the deal -- if the user's SVID is before
-		 * * the start time, and not 0, then check to see
-		 * * if it's a registered account or not.
-		 * *
-		 * * If it IS registered, deal with that accordingly,
-		 * * via handle_burstlogin(). --nenolod
+		 * Here's the deal -- if the user's SVID is before
+		 * the start time, and not 0, then check to see
+		 * if it's a registered account or not.
+		 *
+		 * If it IS registered, deal with that accordingly,
+		 * via handle_burstlogin(). --nenolod
 		 */
 		/* Changed to just check umode +r for now -- jilles */
 #if 0
@@ -581,9 +568,7 @@ static void m_nick(char *origin, uint8_t parc, char *parv[])
 		handle_nickchange(u);
 	}
 
-	/*
-	 * if it's only 2 then it's a nickname change 
-	 */
+	/* if it's only 2 then it's a nickname change */
 	else if (parc == 2)
 	{
 		node_t *n;
@@ -602,22 +587,16 @@ static void m_nick(char *origin, uint8_t parc, char *parv[])
 			/* changed nick to registered one, reset +r */
 			sts(":%s SVSMODE %s +rd %ld", nicksvs.nick, parv[0], time(NULL));
 
-		/*
-		 * remove the current one from the list 
-		 */
+		/* remove the current one from the list */
 		n = node_find(u, &userlist[u->hash]);
 		node_del(n, &userlist[u->hash]);
 		node_free(n);
 
-		/*
-		 * change the nick 
-		 */
+		/* change the nick */
 		strlcpy(u->nick, parv[0], NICKLEN);
 		u->ts = atoi(parv[1]);
 
-		/*
-		 * readd with new nick (so the hash works) 
-		 */
+		/* readd with new nick (so the hash works) */
 		n = node_create();
 		u->hash = UHASH((unsigned char *)u->nick);
 		node_add(u, n, &userlist[u->hash]);
@@ -639,9 +618,7 @@ static void m_quit(char *origin, uint8_t parc, char *parv[])
 {
 	slog(LG_DEBUG, "m_quit(): user leaving: %s", origin);
 
-	/*
-	 * user_delete() takes care of removing channels and so forth 
-	 */
+	/* user_delete() takes care of removing channels and so forth */
 	user_delete(user_find(origin));
 }
 
@@ -670,9 +647,7 @@ static void m_kick(char *origin, uint8_t parc, char *parv[])
 	user_t *u = user_find(parv[1]);
 	channel_t *c = channel_find(parv[0]);
 
-	/*
-	 * -> :rakaur KICK #shrike rintaun :test 
-	 */
+	/* -> :rakaur KICK #shrike rintaun :test */
 	slog(LG_DEBUG, "m_kick(): user was kicked: %s -> %s", parv[1], parv[0]);
 
 	if (!u)
@@ -733,6 +708,13 @@ static void m_server(char *origin, uint8_t parc, char *parv[])
 
 	if (cnt.server == 2)
 		me.actual = sstrdup(parv[0]);
+	else
+	{
+		/* elicit PONG for EOB detection; pinging uplink is
+		 * already done elsewhere -- jilles
+		 */
+		sts(":%s PING %s %s", me.name, me.name, parv[0]);
+	}
 }
 
 static void m_stats(char *origin, uint8_t parc, char *parv[])
@@ -774,9 +756,7 @@ static void m_join(char *origin, uint8_t parc, char *parv[])
 	if (!u)
 		return;
 
-	/*
-	 * JOIN 0 is really a part from all channels 
-	 */
+	/* JOIN 0 is really a part from all channels */
 	if (parv[0][0] == '0')
 	{
 		LIST_FOREACH_SAFE(n, tn, u->channels.head)
@@ -803,9 +783,7 @@ static void m_error(char *origin, uint8_t parc, char *parv[])
 
 void _modinit(module_t * m)
 {
-	/*
-	 * Symbol relocation voodoo. 
-	 */
+	/* Symbol relocation voodoo. */
 	server_login = &ultimate3_server_login;
 	introduce_nick = &ultimate3_introduce_nick;
 	quit_sts = &ultimate3_quit_sts;
